<?php
/*
Plugin Name: W3C Validator (EasyToolsWP)
Description: Validates your pages, posts and products against W3C standards and displays the results in the admin area.
Version: 2.4.1
Author: EasyToolsWP.com
*/

if (!defined('ABSPATH')) {
    exit;
}

add_action('admin_menu', 'w3c_etwp_validator_add_admin_menu');

function w3c_etwp_validator_add_admin_menu() {
    add_menu_page(
        'W3C Validator',
        'W3C Validator',
        'manage_options',
        'w3c-etwp-validator',
        'w3c_etwp_validator_admin_page',
        'dashicons-warning',
        80
    );
}

function w3c_etwp_validator_admin_page() {
    $selected_post_type = isset($_POST['post_type']) ? sanitize_text_field($_POST['post_type']) : 'page';
    $selected_post_id = isset($_POST['post_id']) ? intval($_POST['post_id']) : 0;
    ?>
    <div class="wrap">
        <h1>W3C Validator</h1>
        <div class="w3c-etwp-validator-container">
			
            <div class="w3c-etwp-validator-form">
				<p>
			The W3C standard provides valuable guidance for enhancing your code structure. It is important to note that achieving a 99% score is not expected, as the standard is continually evolving. By adhering to these guidelines, you can achieve improved code quality and compatibility. This plugin, developed by EasyToolsWP, is independent of the W3C organization. The scoring algorithm was also created by EasyToolsWP.
		</p>
                <form method="post" action="">
                    <?php wp_nonce_field('w3c_etwp_validator_nonce_action', 'w3c_etwp_validator_nonce_field'); ?>
                    <label for="post_type"></label>
                    <select name="post_type" id="post_type" onchange="this.form.submit()">
                        <option value="page" <?php selected($selected_post_type, 'page'); ?>>Pages</option>
                        <option value="post" <?php selected($selected_post_type, 'post'); ?>>Posts</option>
                        <option value="product" <?php selected($selected_post_type, 'product'); ?>>Products</option>
                    </select>

                    <br><br>
                    <label for="post_id"></label>
                    <select name="post_id" id="post_id">
                    </select>

                    <br><br>
                    <input type="submit" name="validate_page" class="button button-primary" value="Validate">
                </form>
            </div>
			
			<div class="w3c-etwp-validator-score-container">
                <?php
                if (isset($_POST['validate_page']) && $selected_post_id > 0) {
                    if (!isset($_POST['w3c_etwp_validator_nonce_field']) || !wp_verify_nonce($_POST['w3c_etwp_validator_nonce_field'], 'w3c_etwp_validator_nonce_action')) {
                        echo '<div class="notice notice-error"><p>Invalid nonce.</p></div>';
                        return;
                    }

                    $post_url = get_permalink($selected_post_id);
                    $validation_result = w3c_etwp_validator_validate_page($post_url);

                    if ($validation_result === false) {
                        echo '<div class="notice notice-error"><p>Error with API request.</p></div>';
                    } else {

					$score = 100 - (0.9 * $validation_result['error_count']) - (0.6 * $validation_result['warning_count']) - (0.3 * $validation_result['info_count']);
					$score = max($score, 0);

                        $score_class = '';
                        if ($score >= 85) {
                            $score_class = 'score-excellent';
                        } elseif ($score >= 70) {
                            $score_class = 'score-good';
                        } else {
                            $score_class = 'score-poor';
                        }

                        echo '<div class="w3c-etwp-validator-score ' . esc_attr($score_class) . '">
                                <div class="score-circle">' . esc_html($score) . '</div>
                                <div class="score-bar-container">
                                    <div class="score-bar" style="width: ' . esc_attr($score) . '%;"></div>
                                </div>
                              </div>';
                    }
                }
                ?>
            </div>


        </div>

        <?php
        if (isset($_POST['validate_page']) && $selected_post_id > 0) {
            $post_url = get_permalink($selected_post_id);
            $validation_result = w3c_etwp_validator_validate_page($post_url);

            echo '<div class="w3c-etwp-validator-summary">
                    <h2>Validation Results for: <a href="' . esc_url($post_url) . '" target="_blank">' . esc_html($post_url) . '</a></h2>
                    <div class="w3c-etwp-validator-results">' . $validation_result['output'] . '</div>
                  </div>';
        }
    ?>
    </div>
    <script>
        function fetchPosts(postType) {
            const data = {
                action: 'w3c_etwp_fetch_posts',
                post_type: postType
            };

            jQuery.post(ajaxurl, data, function(response) {
                jQuery('#post_id').html(response);
                <?php if ($selected_post_id > 0): ?>
                    jQuery('#post_id').val(<?php echo $selected_post_id; ?>);
                <?php endif; ?>
            });
        }

        document.addEventListener('DOMContentLoaded', function() {
            fetchPosts('<?php echo esc_js($selected_post_type); ?>');
        });
    </script>
    <style>
        .w3c-etwp-validator-container {
            display: flex;
            align-items: flex-start;
            gap: 20px; /* Space between the form and score container */
        }

        .w3c-etwp-validator-form,
        .w3c-etwp-validator-score-container {
            flex: 1; /* Each takes equal width */
        }

        .w3c-etwp-validator-form {
            max-width:50%;
        }

        .w3c-etwp-validator-score-container {
            text-align: center;
			max-width:250px;
			float:left;
			margin-top:-10px;
        }

        .w3c-etwp-validator-summary {
            clear: both;
            margin-top: 20px;
            padding: 15px;
            border-radius: 5px;
        }

        .w3c-etwp-validator-summary h2 {
            margin-top: 0;
            font-size: 18px;
            color: #333;
        }

        .w3c-etwp-validator-summary a {
            color: #0073aa;
            text-decoration: none;
        }

        .w3c-etwp-validator-summary a:hover {
            text-decoration: underline;
        }

        .w3c-etwp-validator-score {
            text-align: center;
            position: relative;
        }

        .w3c-etwp-validator-score .score-circle {
            font-size: 60px;
            font-weight: bold;
            color: #fff;
            line-height: 80px;
            width: 180px;
            height: 180px;
            border-radius: 50%;
            background-color: #e0e0e0;
            display: flex;
            align-items: center;
            justify-content: center;
            margin-top: 20px;
        }

        .score-excellent .score-circle {
            background-color: #28a745;
        }

        .score-good .score-circle {
            background-color: #ffc107;
        }

        .score-poor .score-circle {
            background-color: #dc3545;
        }

        .w3c-etwp-validator-score .score-bar-container {
            width: 180px;
            height: 20px;
            background-color: #e0e0e0;
            border-radius: 5px;
            margin-top: 20px;
            overflow: hidden;
        }

        .w3c-etwp-validator-score .score-bar {
            height: 100%;
            background-color: #4caf50;
            transition: width 0.3s ease-in-out;
        }

        .score-excellent .score-bar {
            background-color: #28a745;
        }

        .score-good .score-bar {
            background-color: #ffc107;
        }

        .score-poor .score-bar {
            background-color: #dc3545;
        }

        .w3c-etwp-validator-results {
            margin-top: 15px;
        }

        .w3c-etwp-validator-results .w3c-etwp-validator-error,
        .w3c-etwp-validator-results .w3c-etwp-validator-warning,
        .w3c-etwp-validator-results .w3c-etwp-validator-info,
        .w3c-etwp-validator-results .w3c-etwp-validator-code {
            margin-bottom: 20px;
            padding: 15px;
            border-radius: 5px;
            border: 1px solid #ddd;
            background: #f9f9f9;
            box-shadow: 0 0 5px rgba(0, 0, 0, 0.1);
        }

        .w3c-etwp-validator-error {
            background-color: #ff0000;
            border-color: #f5c6cb;
            color: #721c24;
        }

        .w3c-etwp-validator-warning {
            background-color: #ff0000;
            border-color: #ffeeba;
            color: #856404;
        }

        .w3c-etwp-validator-info {
            background-color: #ff0000;
            border-color: #bee5eb;
            color: #0c5460;
        }

        .w3c-etwp-validator-code {
            background: #ffffff;
            border:1px dashed #ced4da!important;
            font-family: monospace;
            font-size: 13px;
            color: #666;
        }

        .w3c-etwp-validator-info-link {
            margin-top: 10px;
            font-size: 12px;
        }

        .w3c-etwp-validator-info-link a {
            color: #0073aa;
            text-decoration: none;
        }

        .w3c-etwp-validator-info-link a:hover {
            text-decoration: underline;
        }
    </style>
    <?php
}

add_action('wp_ajax_w3c_etwp_fetch_posts', 'w3c_etwp_fetch_posts');

function w3c_etwp_fetch_posts() {
    if (isset($_POST['post_type'])) {
        $post_type = sanitize_text_field($_POST['post_type']);
        $posts = get_posts(array(
            'post_type' => $post_type,
            'posts_per_page' => -1
        ));

        foreach ($posts as $post) {
            echo '<option value="' . esc_attr($post->ID) . '">' . esc_html($post->post_title) . '</option>';
        }
    }
    wp_die();
}

function w3c_etwp_validator_validate_page($url) {
    $validator_url = 'https://validator.w3.org/nu/?doc=' . urlencode($url) . '&out=json';
    $response = wp_remote_get($validator_url);
    if (is_wp_error($response)) {
        return false;
    }
    $body = wp_remote_retrieve_body($response);
    $result = json_decode($body, true);

    $errors = 0;
    $warnings = 0;
    $infos = 0;
    $output = '';

    if (!empty($result['messages'])) {
        foreach ($result['messages'] as $message) {
            $type = $message['type'];
            $message_text = $message['message'];
            $extract = isset($message['extract']) ? $message['extract'] : '';
            $first_line = isset($message['lastLine']) ? ' (Line ' . $message['lastLine'] . ')' : '';
            $first_column = isset($message['lastColumn']) ? ', Column ' . $message['lastColumn'] : '';

            $info_link = '<a href="https://validator.w3.org/nu/?doc=' . urlencode($url) . '&out=json" target="_blank">More info</a>';

            if ($type == 'error') {
                $errors++;
                $output .= '<div class="w3c-etwp-validator-error"><strong>Error' . $first_line . $first_column . ':</strong> ' . esc_html($message_text);
                if ($extract) {
                    $output .= '<div class="w3c-etwp-validator-code">' . esc_html($extract) . '</div>';
                }
                $output .= '<div class="w3c-etwp-validator-info-link">' . $info_link . '</div>';
                $output .= '</div>';
            } elseif ($type == 'warning') {
                $warnings++;
                $output .= '<div class="w3c-etwp-validator-warning"><strong>Warning' . $first_line . $first_column . ':</strong> ' . esc_html($message_text);
                if ($extract) {
                    $output .= '<div class="w3c-etwp-validator-code">' . esc_html($extract) . '</div>';
                }
                $output .= '<div class="w3c-etwp-validator-info-link">' . $info_link . '</div>';
                $output .= '</div>';
            } else {
                $infos++;
                $output .= '<div class="w3c-etwp-validator-info"><strong>Info' . $first_line . $first_column . ':</strong> ' . esc_html($message_text);
                if ($extract) {
                    $output .= '<div class="w3c-etwp-validator-code">' . esc_html($extract) . '</div>';
                }
                $output .= '<div class="w3c-etwp-validator-info-link">' . $info_link . '</div>';
                $output .= '</div>';
            }
        }
    }

    return array(
        'error_count' => $errors,
        'warning_count' => $warnings,
        'info_count' => $infos,
        'output' => $output
    );
}
?>